/*
 * Created on 2005/04/18
 */
package roomlayout;

import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.SwingUtilities;
import javax.swing.event.MouseInputListener;

/**
 * @author kwsk
 */
public class RoomController
implements MouseInputListener, MouseWheelListener
{
    private RoomModel model;
    private RoomView view;
    private JFrame frame;
    private ControlDialog controlDialog;
    
    public RoomController(RoomModel model, RoomView view, JFrame frame) {
        this.model = model;

        this.view = view;
        view.addMouseListener(this);
        view.addMouseMotionListener(this);
        view.addMouseWheelListener(this);

        this.frame = frame;
        JMenuBar menuBar = getMenuBar();
        frame.setJMenuBar(menuBar);
        
        this.controlDialog = new ControlDialog(model, view);
    }

    ///////////////////////////////////////////////
    // }EXCxg֘A
    
    Piece target = null;
    int ox = 0, oy = 0;
    int ix = 0, iy = 0, ia = 0;
    boolean rotation = false;
    
    public void mouseClicked(MouseEvent e) {
    }
    public void mouseEntered(MouseEvent e) {
    }
    public void mouseExited(MouseEvent e) {
    }
    public void mousePressed(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();
        target = model.pieceAt(x, y);
        view.setTarget(target);
        if(target != null) {
            ox = x;
            oy = y;
        }
        view.repaint();
    }
    public void mouseReleased(MouseEvent e) {
    }
    public void mouseDragged(MouseEvent e) {
        int x = e.getX();
        int y = e.getY();
        if(target == null) return;
        int dx = x - ox;
        int dy = y - oy;
        target.translate(dx, dy);
        
        int r = target.w / 2 + target.h / 2 + Math.abs(dx) + Math.abs(dy); // L߂
        view.repaint(new Rectangle(target.x - r, target.y -r, r * 2, r * 2));
        ox = x;
        oy = y;
    }
    public void mouseMoved(MouseEvent e) {
    }
    public void mouseWheelMoved(MouseWheelEvent e) {
    }


    ///////////////////////////////////
    // j[֘A
    
    private JMenuBar getMenuBar() {
        JMenuBar menuBar = new JMenuBar();
        
        JMenu menuFile = new JMenu("File");
        menuBar.add(menuFile);
        JMenuItem menuItemNew = new JMenuItem(new NewAction());
        menuFile.add(menuItemNew);
        JMenuItem menuItemLoad = new JMenuItem(new LoadAction());
        menuFile.add(menuItemLoad);
        JMenuItem menuItemSave = new JMenuItem(new SaveAction());
        menuFile.add(menuItemSave);
        return menuBar;
    }
    
    public class NewAction extends AbstractAction {
        public NewAction() {
            super("New");
        }
        public void actionPerformed(ActionEvent e) {
            model.init();
            view.repaint();
        }
    }
    
    public class LoadAction extends AbstractAction {
        public LoadAction() {
            super("Load");
        }
        public void actionPerformed(ActionEvent e) {
            JFileChooser fileChooser = new JFileChooser("data");
            String[] extensions = {"txt"};
            ExampleFileFilter filter = new ExampleFileFilter(extensions, ".txt");
            fileChooser.addChoosableFileFilter(filter);
            
            int returnVal = fileChooser.showOpenDialog(view);
            if(returnVal != JFileChooser.APPROVE_OPTION) return;
            File file = fileChooser.getSelectedFile();
            model.load(file);
            view.repaint();
        }
    }

    public class SaveAction extends AbstractAction {
        public SaveAction() {
            super("Save");
        }
        public void actionPerformed(ActionEvent e) {
            JFileChooser fileChooser = new JFileChooser("data");
            String[] extensions = {"txt"};
            ExampleFileFilter filter = new ExampleFileFilter(extensions, ".txt");
            fileChooser.addChoosableFileFilter(filter);

            int returnVal = fileChooser.showSaveDialog(view);
            if(returnVal != JFileChooser.APPROVE_OPTION) return;
            File file = fileChooser.getSelectedFile();
            model.save(file);

            File imageFile = new File("data/" + file.getName() + ".png");
            view.saveAsImage(imageFile);
            
            view.repaint();
        }
    }

}
