/*
 * Created on 2005/04/18
 */
package roomlayout;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Iterator;

import javax.imageio.ImageIO;
import javax.swing.JPanel;

/**
 * @author kwsk
 */
public class RoomView extends JPanel
{
    private RoomModel model;
    private int gridInterval = 50;

    private Piece target = null;
    
    
    public RoomView(RoomModel model) {
        this.model = model;
        setPreferredSize(new Dimension(800, 600));
        setBackground(Color.white);
    }
    
    protected void paintComponent(Graphics _g) {
        super.paintComponent(_g);
        Graphics2D g = (Graphics2D)_g;
        
        g.setFont(g.getFont().deriveFont(24F));
        
        drawGrid(g);
        
        synchronized(model) {
            for(Iterator i = model.pieceIterator() ; i.hasNext() ; ) {
                Piece p = (Piece)i.next();
                if(p != target) drawPiece(g, p, false);
            }
            
            // IĂ͍Ōɕ`
            if(target != null && model.containsPiece(target)) {
                drawPiece(g, target, true);
            }
        }
    }

    
    private Stroke stroke = new BasicStroke(2);
    private float[] dash = {5f, 8f};
    private Stroke strokeSelected = new BasicStroke(2, BasicStroke.CAP_SQUARE, BasicStroke.JOIN_MITER, 10.0f, dash, 0.0f);
    
    private void drawPiece(Graphics2D g, Piece p, boolean selected) {
        AffineTransform transform = g.getTransform(); // ޔ
        g.translate(p.x, p.y);
        g.rotate(Math.toRadians(p.getAngle()));

        // 
        g.setColor(p.getColor());
        g.fillRect(- p.w/2, - p.h/2, p.w, p.h);

        // ӂ
        g.setStroke(selected ? strokeSelected : stroke); // IĂ͔j
        g.setColor(p.getColor().darker());
        g.drawRect(- p.w/2, - p.h/2, p.w, p.h);

        // ID
        g.setTransform(transform); // 
        g.translate(p.x, p.y);
        drawPieceId(g, p);
        
        g.setTransform(transform); // 
    }

    
    private FontMetrics fm = null; // cache
    
    private void drawPieceId(Graphics2D g, Piece p) {
        g.setColor(Color.black);
        int id = model.getId(p);
        String s = Integer.toString(id);
        if(fm == null) fm = g.getFontMetrics();
        int offsetX = - fm.stringWidth(s) / 2;
        int offsetY = fm.getHeight() / 2 - fm.getDescent();
        g.drawString(s, offsetX, offsetY);
        if(id == 9) g.drawString("_", offsetX, offsetY + 2); // ad hoc
    }
    
    private void drawGrid(Graphics g) {
        g.setColor(Color.lightGray);
        int w = getWidth();
        int h = getHeight();
        for(int x = 0 ; x < w ; x += gridInterval) {
            g.drawLine(x, 0, x, h);
        }
        for(int y = 0 ; y < h ; y += gridInterval) {
            g.drawLine(0, y, w, y);
        }
    }
    
    public void setTarget(Piece p) {
        target = p;
    }

    // R|[lg̕\摜Ƃĕۑ
    public void saveAsImage(File file) {
        BufferedImage image = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_4BYTE_ABGR);
        Graphics g = image.getGraphics();
        paint(g);
        try {
            ImageIO.write(image, "png", file);
        }
        catch(IOException e) {
            e.printStackTrace();
        }
    }

}


